Ext.namespace("dSS", "dSS.util");

dSS.util.shortDSID = function(dsid) {
    if (dsid.length > 8) {
        return dsid.slice(dsid.length - 8);
    } else {
        return dsid;
    }
};

dSS.util.getName = function(name) {
    if ((name === undefined) || (name.length === 0)) {
        return '<i>' + dSS._("no name") + '</i>';
    } else {
        return name;
    }
};

dSS.util.decode16 = function(version) {
    var low =  (version & 0x000000ff);
    var high = (version & 0x0000ff00) >> 8;
    return String(high) + "." + low;
};

dSS.util.decode32 = function(version) {
    var v1 = (version & 0xff000000) >> 24;
    var v2 = (version & 0x00ff0000) >> 16;
    var v3 = (version & 0x0000ff00) >> 8;
    var v4 = (version & 0x000000ff);

    return String(v1) + "." + v2  + "." + v3 + "." + v4;
};

dSS.util.parseFunctionID = function(functionID) {
    var dSclass = (functionID & 0xf000) >> 12;
    var dSsubclass = (functionID & 0xfc0) >> 6;
    var dSfuncModule = (functionID & 0x3f);

    return [dSclass, dSsubclass, dSfuncModule];
};

dSS.util.parseProductID = function(productID) {
    var deviceType = (productID >> 10) & 0x3f;
    var deviceNumber = productID & 0x3ff;
    return [deviceType, deviceNumber];
};

dSS.util.deviceStringFromProduct = function(functionID, productID) {
    var fid = dSS.util.parseFunctionID(functionID);
    var devShort = dSS.config.deviceColors[fid[0]];
    if (devShort === undefined) {
        devShort = "";
    } else {
        devShort = devShort + "-";
    }

    var result = dSS.util.parseProductID(productID);
    if (result.length < 2) {
        return "";
    }

    var devType = dSS.config.deviceTypes[result[0]];
    if (devType === undefined) {
        devType = "??";
    }

    var devNumber = result[1];
    if (devNumber === undefined) {
        devNumber = "???";
    }

    return devShort + devType + " " + result[1];

};

// returns object with following configuration parameters:
// dontCare: don't care flag
// ignoreLocalPriorisation: ignore local priorisation
// blink: blink
// ledConfig: led config
// dimmConfig: dim time config
dSS.util.decodeSceneConfig = function(sc) {
    var ret = {};
    ret.dontCare = (sc & 0x01);                     // bit 0
    ret.ignoreLocalPriorisation = (sc & 0x02) >> 1; // bit 1
    ret.blink = (sc & 0x08) >> 3;                   // bit 3
    ret.ledConfig = (sc & 0x30) >> 4;               // bit 4-5
    ret.dimConfig = (sc & 0xc0) >> 6;               // bit 6-7

    return ret;
};

// parameter: configuration object as provided by the decodeSceneConfig function
dSS.util.encodeSceneConfig = function(config) {
    return (config.dontCare & 0x01)                         // bit 0
        | ((config.ignoreLocalPriorisation << 1) & 0x02)    // bit 1
        | ((config.blink << 3) & 0x08)                      // bit 3
        | ((config.ledConfig << 4) & 0x30)                  // bit 4-5
        | ((config.dimConfig << 6) & 0xc0);                 // bit 6-7
};

dSS.util.dec2hex = function(d) {
  var hD = '0123456789ABCDEF';
  var h = hD.substr(d & 15, 1);
  while (d > 15) {
    d>>=4;
    h = hD.substr(d & 15, 1) + h;
  }
  return h;
};

dSS.util.callScene = function(dsid, sceneNumber) {
    Ext.Ajax.request({
        url: '/json/device/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            dsid: dsid,
            sceneNumber: sceneNumber
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"), dSS._("Could not call scene")
                                  + ': ' + jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.callSceneOnZone = function(zoneId, sceneNumber, groupId, force) {
    if (force === undefined) {
        force = false;
    }
    Ext.Ajax.request({
        url: '/json/zone/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId,
            force: force
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"), dSS._("Could not call scene")
                                  + ': ' + jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.saveSceneOnZone = function(zoneId, sceneNumber, groupId) {
    Ext.Ajax.request({
        url: '/json/zone/saveScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.MessageBox.alert(dSS._("Could not save scene") + " " + sceneNumber, jsonData.message);
                } else {
                    var stmp = dSS._(dSS.config.sceneNames[sceneNumber]);
                    var ztmp = dSS._("in zone") + ' &quot;' + '#' + zoneId +
                               '&quot; ';

                    var s = dSS.data.Loader.getSceneStore();
                    var srec = s.getById(zoneId + '-' + groupId + '-' +
                                         sceneNumber);
                    if (srec !== undefined) {
                        stmp = dSS.util.getName(srec.get('name'));
                    }

                    if (zoneId === 0) {
                        ztmp = '';
                    } else {
                        var z = dSS.data.Loader.getZoneStore();
                        var zrec = z.getById(zoneId);
                        if (zrec !== undefined) {
                            ztmp = dSS._("in zone") + ' &quot;' +
                                   dSS.util.getName(zrec.get('name')) +
                                   '&quot; ';
                        }
                    }

                    var msg = dSS._("Scene") + ' &quot;' + stmp + '&quot; ' +
                              ztmp +  dSS._("was saved");

                    dSS.MessageBus.fireEvent('statusmessage', msg);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send saveScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send saveScene command to dSS"));
        }
    });
};

dSS.util.setValueOnDevice = function(dsid, value) {
    Ext.Ajax.request({
        url: '/json/device/setValue',
        disableCaching: true,
        method: "GET",
        params: {
            dsid: dsid,
            value: value
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not set value") + ": " +
                                  jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send setValue command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send setValue command to dSS"));
        }
    });

};

/* parameter object
    {
        url, // url string
        params // ajax request parameters
        failureMsg // failure message to show in alert messagebox
        maskEl // element to mask during loading
        maskMsg // mask message
        scope // object scope (passt to ajax requuest)
        evt // event name (string) to fire
    }
    returns AjaxRequest object cotnainig the transaction id
*/
dSS.util.genericRequest = function(param, callback) {
    if (param === undefined) {
        throw dSS._("Call with undefined parameter");
    }

    if (param.maskEl !== undefined) {
        if (param.msgMsg !== undefined) {
            param.maskEl.mask(param.maskMsg + '...', 'x-mask-loading');
        } else {
            param.maskEl.mask();
        }
    }

    return Ext.Ajax.request(
    {
        url: param.url,
        method: 'GET',
        params: param.params,
        scope: param.scope,
        success: function(response) {
            var ok = false;
            try {
                var jsonData = Ext.util.JSON.decode(
                                    response.responseText);
                if (jsonData.ok !== true) {
                    if (param.failureMsg !== undefined) {
                        Ext.Msg.alert(dSS._("Error"), param.failureMsg + ': ' +
                                      jsonData.message);
                    } else {
                        Ext.Msg.alert(dSS._("Error"), jsonData.message);
                    }
                } else {
                    ok = true;
                }
            } catch (error) {
                Ext.Msg.alert(dSS._("Error"), error);
            }

            if (param.maskEl !== undefined) {
                param.maskEl.unmask();
            }

            if (callback !== undefined) {
                callback(ok);
            }

            if (param.evt !== undefined) {
                this.fireEvent(param.evt); // scope set by ajax request
            }
        },
        failure: function(result, request) {
            var msg;
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (param.failureMsg !== undefined) {
                    msg = param.failureMsg + ': ' + jsonData.message;
                } else {
                    msg = jsonData.message;
                }
            } catch (err) {
                if (param.failureMsg !== undefined) {
                    msg = param.failureMsg + ': ' + err;
                } else {
                    msg = err;
                }
            }

            Ext.Msg.alert(dSS._("Error"), msg);
            if (param.maskEl !== undefined) {
                param.maskEl.unmask();
            }

            if (callback !== undefined) {
                callback(false);
            }

            if (param.evt !== undefined) {
                this.fireEvent(param.evt);
            }
        }
    });
};

dSS.util.getJokerGroup = function(groups) {
    var i;
    var group = null;
    for (i = 0; i < groups.length; i++) {
        group = parseInt(groups[i], 10);
        if (group !== dSS.config.groups.JOKER) {
            return group;
        }
    }
    return null;
};

dSS.util.updateJokerGroup = function(groups, oldGroup, newGroup) {
    var i;
    for (i = 0; i < groups.length; i++) {
        if (String(oldGroup) === groups[i]) {
            groups[i] = String(newGroup);
        }
    }
    return groups;
};

// device type, device class, return window icon (default false)
dSS.util.getDeviceIconCSS = function(type, cls, title, groups) {
    var css;
    var joker;

    if (groups === undefined) {
        groups = [];
    }

    if (cls === dSS.config.groups.JOKER) {
        var group = dSS.util.getJokerGroup(groups);
        if (group !== null) {
            joker = '_' + dSS.config.deviceColors[group];
        }
    }

    try {
        css = dSS.config.DeviceIconCSS[type][cls];
        if (joker !== undefined) {
            css = css + joker;
        }
    } catch (err) {
        if (cls !== undefined) {
            css = dSS.config.GenericDeviceIconCls[cls];
        }
    }

    if ((css !== undefined) && (title === true)) {
        css = css + dSS.config.TitleIconCSSExt;
    }

    return css;
};

dSS.util.UniqueArray = function(array, ints) {
  var i;
  var length = array.length;
  var helper = {};
  var result = [];

  for (i = 0; i < length; i++) {
      helper[array[i]] = 0;
  }

  for (i in helper) {
      if (ints === true) {
          result.push(parseInt(i, 10));
      } else {
          result.push(i);
      }
  }

  return result;
};
